glycowork
================

<!-- WARNING: THIS FILE WAS AUTOGENERATED! DO NOT EDIT! -->

![CI](https://github.com/BojarLab/glycowork/workflows/CI/badge.svg)
[![HitCount](https://hits.dwyl.com/BojarLab/glycowork.svg?style=flat-square)](http://hits.dwyl.com/BojarLab/glycowork)
[![contributions
welcome](https://img.shields.io/badge/contributions-welcome-brightgreen.svg?style=flat)](https://github.com/dwyl/esta/issues)

<img src="./glycowork_badge_wo_bg.jpg" width="200" />

Glycans are a fundamental biological sequence, similar to DNA, RNA, or
proteins. Glycans are complex carbohydrates that can form branched
structures comprising monosaccharides and linkages as constituents.
Despite being conspicuously absent from most research, glycans are
ubiquitous in biology. They decorate most proteins and lipids and direct
the stability and functions of biomolecules, cells, and organisms. This
also makes glycans relevant to every human disease.

The analysis of glycans is made difficult by their nonlinearity and
their astounding diversity, given the large number of monosaccharides
and types of linkages. `Glycowork` is a Python package designed to
process and analyze glycan sequences, with a special emphasis on
glycan-focused data science and machine learning. Next to various
functions to work with glycans, `glycowork` also contains glycan data
that can be used for model training, motif comparisons, etc. Glycowork
uses human-readable IUPAC-condensed (e.g.,
`Neu5Ac(a2-3)Gal(b1-3)[Neu5Ac(a2-6)]GalNAc`) as a user-facing string
representation and monosaccharide-level graphs as a powerful
behind-the-scenes architecture.

If you use `glycowork` or any of our datasets in your project, please
cite [Thomes et al.,
2021](https://academic.oup.com/glycob/advance-article/doi/10.1093/glycob/cwab067/6311240).

The full documentation for `glycowork` can be found here:
https://bojarlab.github.io/glycowork/

If you want to contribute to `glycowork`, the best place to start is to
read our [contribution
guidelines](https://github.com/BojarLab/glycowork/blob/master/CONTRIBUTING.md).

## Install

via pip: <br> `pip install glycowork` <br> `import glycowork`

alternative: <br>
`pip install git+https://github.com/BojarLab/glycowork.git` <br>
`import glycowork`

Note that we have optional extra installs for specialized use (even
further instructions can be found in the `Examples` tab), such as: <br>
*deep learning* <br> `pip install glycowork[ml]` <br> *drawing glycan
images with GlycoDraw* <br> `pip install glycowork[draw]` <br>
*everything* <br> `pip install glycowork[all]` <br>

## Data & Models

`Glycowork` currently contains the following main datasets that are
freely available to everyone:

- **`df_glycan`**
  - contains \~50,000 unique glycan sequences, including labels such as
    \~35,000 species associations, \~14,000 tissue associations, and
    \~1,000 disease associations
- **`glycan_binding`**
  - contains \>550,000 protein-glycan binding interactions, from 1,392
    unique glycan-binding proteins

Additionally, we store these trained deep learning models for easy
usage, which can be retrieved with the `prep_model` function:

- **`LectinOracle`**
  - can be used to predict glycan-binding specificity of a protein,
    given its ESM-1b representation; from [Lundstrom et al.,
    2021](https://onlinelibrary.wiley.com/doi/10.1002/advs.202103807)
- **`LectinOracle_flex`**
  - operates the same as LectinOracle but can directly use the raw
    protein sequence as input (no ESM-1b representation required)
- **`SweetNet`**
  - a graph convolutional neural network trained to predict species from
    glycan, can be used to generate learned glycan representations; from
    [Burkholz et al., 2021](https://pubmed.ncbi.nlm.nih.gov/34133929/)
- **`NSequonPred`**
  - given the ESM-1b representation of an N-sequon (+/- 20 AA), this
    model can predict whether the sequon will be glycosylated

## How to use

`Glycowork` currently contains four main modules:

- **`glycan_data`**
  - stores several glycan datasets and contains helper functions
- **`ml`**
  - here are all the functions for training and using machine learning
    models, including train-test-split, getting glycan representations,
    etc.
- **`motif`**
  - contains functions for processing & drawing glycan sequences,
    identifying motifs and features, and analyzing them
- **`network`**
  - contains functions for constructing and analyzing glycan networks
    (e.g., biosynthetic networks)

Below are some examples of what you can do with `glycowork`; be sure to
check out the other `examples` in the full documentation for everything
that’s there. A non-exhaustive list includes:

- using trained AI models for prediction
- training your own AI models
- motif enrichment analyses
- annotating motifs in glycans
- drawing publication-quality glycan figures
- finding out whether & where glycans are describing the same sequence
- m/z to composition to structure to motif mappings
- mass calculation
- visualizing motif distribution / glycan similarities / sequence
  properties
- constructing and analyzing biosynthetic networks

``` python
#drawing publication-quality glycan figures
from glycowork.motif.draw import GlycoDraw
GlycoDraw("Neu5Ac(a2-3)Gal(b1-4)[Fuc(a1-3)]GlcNAc(b1-2)Man(a1-3)[Neu5Gc(a2-6)Gal(b1-4)GlcNAc(b1-2)Man(a1-6)][GlcNAc(b1-4)]Man(b1-4)GlcNAc(b1-4)[Fuc(a1-6)]GlcNAc")
```

![](index_files/figure-commonmark/cell-3-output-1.svg)

``` python
#get motifs, graph features, and sequence features of a set of glycan sequences to train models or analyze glycan properties
glycans = ['Man(a1-3)[Man(a1-6)][Xyl(b1-2)]Man(b1-4)GlcNAc(b1-4)[Fuc(a1-3)]GlcNAc',
           'Man(a1-2)Man(a1-2)Man(a1-3)[Man(a1-3)Man(a1-6)]Man(b1-4)GlcNAc(b1-4)GlcNAc',
           'GalNAc(a1-4)GlcNAcA(a1-4)[GlcN(b1-7)]Kdo(a2-5)[Kdo(a2-4)]Kdo(a2-6)GlcN4P(b1-6)GlcN1P']
from glycowork.motif.annotate import annotate_dataset
out = annotate_dataset(glycans, feature_set = ['known', 'graph', 'exhaustive']).head()
```

<table border="1" class="dataframe">
  <thead>
    <tr style="text-align: right;">
      <th></th>
      <th>LewisX</th>
      <th>LewisY</th>
      <th>SialylLewisX</th>
      <th>SulfoSialylLewisX</th>
      <th>LewisA</th>
      <th>LewisB</th>
      <th>SialylLewisA</th>
      <th>SulfoLewisA</th>
      <th>H_type2</th>
      <th>H_type1</th>
      <th>A_antigen</th>
      <th>B_antigen</th>
      <th>Galili_antigen</th>
      <th>GloboH</th>
      <th>Gb5</th>
      <th>Gb4</th>
      <th>Gb3</th>
      <th>3SGb3</th>
      <th>8DSGb3</th>
      <th>3SGb4</th>
      <th>8DSGb4</th>
      <th>6DSGb4</th>
      <th>3SGb5</th>
      <th>8DSGb5</th>
      <th>6DSGb5</th>
      <th>6DSGb5_2</th>
      <th>6SGb3</th>
      <th>8DSGb3_2</th>
      <th>6SGb4</th>
      <th>8DSGb4_2</th>
      <th>6SGb5</th>
      <th>8DSGb5_2</th>
      <th>66DSGb5</th>
      <th>Forssman_antigen</th>
      <th>iGb3</th>
      <th>I_antigen</th>
      <th>i_antigen</th>
      <th>PI_antigen</th>
      <th>Chitobiose</th>
      <th>Trimannosylcore</th>
      <th>Internal_LacNAc_type1</th>
      <th>Terminal_LacNAc_type1</th>
      <th>Internal_LacNAc_type2</th>
      <th>Terminal_LacNAc_type2</th>
      <th>Internal_LacdiNAc_type1</th>
      <th>Terminal_LacdiNAc_type1</th>
      <th>Internal_LacdiNAc_type2</th>
      <th>Terminal_LacdiNAc_type2</th>
      <th>bisectingGlcNAc</th>
      <th>VIM</th>
      <th>PolyLacNAc</th>
      <th>Ganglio_Series</th>
      <th>Lacto_Series(LewisC)</th>
      <th>NeoLacto_Series</th>
      <th>betaGlucan</th>
      <th>KeratanSulfate</th>
      <th>Hyluronan</th>
      <th>Mollu_series</th>
      <th>Arthro_series</th>
      <th>Cellulose_like</th>
      <th>Chondroitin_4S</th>
      <th>GPI_anchor</th>
      <th>Isoglobo_series</th>
      <th>LewisD</th>
      <th>Globo_series</th>
      <th>Sda</th>
      <th>SDA</th>
      <th>Muco_series</th>
      <th>Heparin</th>
      <th>Peptidoglycan</th>
      <th>Dermatansulfate</th>
      <th>CAD</th>
      <th>Lactosylceramide</th>
      <th>Lactotriaosylceramide</th>
      <th>LexLex</th>
      <th>GM3</th>
      <th>H_type3</th>
      <th>GM2</th>
      <th>GM1</th>
      <th>cisGM1</th>
      <th>VIM2</th>
      <th>GD3</th>
      <th>GD1a</th>
      <th>GD2</th>
      <th>GD1b</th>
      <th>SDLex</th>
      <th>Nglycolyl_GM2</th>
      <th>Fuc_LN3</th>
      <th>GT1b</th>
      <th>GD1</th>
      <th>GD1a_2</th>
      <th>LcGg4</th>
      <th>GT3</th>
      <th>Disialyl_T_antigen</th>
      <th>GT1a</th>
      <th>GT2</th>
      <th>GT1c</th>
      <th>2Fuc_GM1</th>
      <th>GQ1c</th>
      <th>O_linked_mannose</th>
      <th>GT1aa</th>
      <th>GQ1b</th>
      <th>HNK1</th>
      <th>GQ1ba</th>
      <th>O_mannose_Lex</th>
      <th>2Fuc_GD1b</th>
      <th>Sialopentaosylceramide</th>
      <th>Sulfogangliotetraosylceramide</th>
      <th>B-GM1</th>
      <th>GQ1aa</th>
      <th>bisSulfo-Lewis x</th>
      <th>para-Forssman</th>
      <th>core_fucose</th>
      <th>core_fucose(a1-3)</th>
      <th>GP1c</th>
      <th>B-GD1b</th>
      <th>GP1ca</th>
      <th>Isoglobotetraosylceramide</th>
      <th>polySia</th>
      <th>high_mannose</th>
      <th>Gala_series</th>
      <th>LPS_core</th>
      <th>Nglycan_complex</th>
      <th>Nglycan_complex2</th>
      <th>Oglycan_core1</th>
      <th>Oglycan_core2</th>
      <th>Oglycan_core3</th>
      <th>Oglycan_core4</th>
      <th>Oglycan_core5</th>
      <th>Oglycan_core6</th>
      <th>Oglycan_core7</th>
      <th>Xylogalacturonan</th>
      <th>Sialosylparagloboside</th>
      <th>LDNF</th>
      <th>OFuc</th>
      <th>Arabinogalactan_type2</th>
      <th>EGF_repeat</th>
      <th>Nglycan_hybrid</th>
      <th>Arabinan</th>
      <th>Xyloglucan</th>
      <th>Acharan_Sulfate</th>
      <th>M3FX</th>
      <th>M3X</th>
      <th>1-6betaGalactan</th>
      <th>Arabinogalactan_type1</th>
      <th>Galactomannan</th>
      <th>Tetraantennary_Nglycan</th>
      <th>Mucin_elongated_core2</th>
      <th>Fucoidan</th>
      <th>Alginate</th>
      <th>FG</th>
      <th>XX</th>
      <th>Difucosylated_core</th>
      <th>GalFuc_core</th>
      <th>diameter</th>
      <th>branching</th>
      <th>nbrLeaves</th>
      <th>avgDeg</th>
      <th>varDeg</th>
      <th>maxDeg</th>
      <th>nbrDeg4</th>
      <th>max_deg_leaves</th>
      <th>mean_deg_leaves</th>
      <th>deg_assort</th>
      <th>betweeness</th>
      <th>betwVar</th>
      <th>betwMax</th>
      <th>eigenMax</th>
      <th>eigenMin</th>
      <th>eigenAvg</th>
      <th>eigenVar</th>
      <th>closeMax</th>
      <th>closeMin</th>
      <th>closeAvg</th>
      <th>closeVar</th>
      <th>flowMax</th>
      <th>flowAvg</th>
      <th>flowVar</th>
      <th>flow_edgeMax</th>
      <th>flow_edgeMin</th>
      <th>flow_edgeAvg</th>
      <th>flow_edgeVar</th>
      <th>loadMax</th>
      <th>loadAvg</th>
      <th>loadVar</th>
      <th>harmMax</th>
      <th>harmMin</th>
      <th>harmAvg</th>
      <th>harmVar</th>
      <th>secorderMax</th>
      <th>secorderMin</th>
      <th>secorderAvg</th>
      <th>secorderVar</th>
      <th>size_corona</th>
      <th>size_core</th>
      <th>nbr_node_types</th>
      <th>egap</th>
      <th>entropyStation</th>
      <th>N</th>
      <th>dens</th>
      <th>GlcN</th>
      <th>Man</th>
      <th>Fuc</th>
      <th>GlcN4P</th>
      <th>GlcNAcA</th>
      <th>Kdo</th>
      <th>Xyl</th>
      <th>GalNAc</th>
      <th>GlcNAc</th>
      <th>GlcN1P</th>
      <th>Kdo(a2-6)GlcN4P</th>
      <th>GlcN4P(b1-6)GlcN1P</th>
      <th>GlcNAcA(a1-4)Kdo</th>
      <th>Xyl(b1-2)Man</th>
      <th>GlcNAc(b1-4)GlcNAc</th>
      <th>GalNAc(a1-4)GlcNAcA</th>
      <th>GlcN(b1-7)Kdo</th>
      <th>Fuc(a1-3)GlcNAc</th>
      <th>Kdo(a2-5)Kdo</th>
      <th>Man(b1-4)GlcNAc</th>
      <th>Kdo(a2-4)Kdo</th>
      <th>Man(a1-6)Man</th>
      <th>Man(a1-3)Man</th>
      <th>Man(a1-2)Man</th>
    </tr>
  </thead>
  <tbody>
    <tr>
      <th>Man(a1-3)[Man(a1-6)][Xyl(b1-2)]Man(b1-4)GlcNAc(b1-4)[Fuc(a1-3)]GlcNAc</th>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>1</td>
      <td>1</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>1</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>8.0</td>
      <td>1.0</td>
      <td>4.0</td>
      <td>1.846154</td>
      <td>0.591716</td>
      <td>4.0</td>
      <td>1.0</td>
      <td>4.0</td>
      <td>4.0</td>
      <td>-3.448276e-02</td>
      <td>0.240093</td>
      <td>0.051241</td>
      <td>0.727273</td>
      <td>0.337083</td>
      <td>0.251423</td>
      <td>0.276471</td>
      <td>0.000487</td>
      <td>0.400000</td>
      <td>0.181818</td>
      <td>0.288591</td>
      <td>0.003994</td>
      <td>0.727273</td>
      <td>0.240093</td>
      <td>0.051241</td>
      <td>0.318182</td>
      <td>0.090909</td>
      <td>0.179293</td>
      <td>0.006460</td>
      <td>0.727273</td>
      <td>0.240093</td>
      <td>0.051241</td>
      <td>6.950000</td>
      <td>3.253571</td>
      <td>4.820330</td>
      <td>0.908780</td>
      <td>66.603303</td>
      <td>26.305893</td>
      <td>44.589784</td>
      <td>127.751200</td>
      <td>4.0</td>
      <td>13.0</td>
      <td>13.0</td>
      <td>0.057103</td>
      <td>-2.075921</td>
      <td>13.0</td>
      <td>12.0</td>
      <td>2</td>
      <td>3</td>
      <td>1</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>1</td>
      <td>0</td>
      <td>2</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>1</td>
      <td>1</td>
      <td>0</td>
      <td>0</td>
      <td>1</td>
      <td>0</td>
      <td>1</td>
      <td>0</td>
      <td>1</td>
      <td>1</td>
      <td>0</td>
    </tr>
    <tr>
      <th>Man(a1-2)Man(a1-2)Man(a1-3)[Man(a1-3)Man(a1-6)]Man(b1-4)GlcNAc(b1-4)GlcNAc</th>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>1</td>
      <td>1</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>10.0</td>
      <td>1.0</td>
      <td>3.0</td>
      <td>1.866667</td>
      <td>0.248889</td>
      <td>3.0</td>
      <td>0.0</td>
      <td>3.0</td>
      <td>3.0</td>
      <td>-2.331468e-15</td>
      <td>0.263004</td>
      <td>0.037282</td>
      <td>0.703297</td>
      <td>0.288267</td>
      <td>0.234925</td>
      <td>0.257853</td>
      <td>0.000179</td>
      <td>0.341463</td>
      <td>0.157303</td>
      <td>0.238951</td>
      <td>0.003059</td>
      <td>0.703297</td>
      <td>0.263004</td>
      <td>0.037282</td>
      <td>0.296703</td>
      <td>0.076923</td>
      <td>0.182104</td>
      <td>0.005067</td>
      <td>0.703297</td>
      <td>0.263004</td>
      <td>0.037282</td>
      <td>6.616667</td>
      <td>3.407937</td>
      <td>4.921958</td>
      <td>0.799295</td>
      <td>70.823725</td>
      <td>26.381812</td>
      <td>48.985176</td>
      <td>174.452553</td>
      <td>3.0</td>
      <td>15.0</td>
      <td>15.0</td>
      <td>0.031066</td>
      <td>-2.658737</td>
      <td>15.0</td>
      <td>14.0</td>
      <td>2</td>
      <td>6</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>2</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>1</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>1</td>
      <td>0</td>
      <td>1</td>
      <td>1</td>
      <td>1</td>
    </tr>
    <tr>
      <th>GalNAc(a1-4)GlcNAcA(a1-4)[GlcN(b1-7)]Kdo(a2-5)[Kdo(a2-4)]Kdo(a2-6)GlcN4P(b1-6)GlcN1P</th>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
      <td>10.0</td>
      <td>2.0</td>
      <td>4.0</td>
      <td>1.866667</td>
      <td>0.382222</td>
      <td>3.0</td>
      <td>0.0</td>
      <td>4.0</td>
      <td>4.0</td>
      <td>-1.449275e-02</td>
      <td>0.239560</td>
      <td>0.044684</td>
      <td>0.615385</td>
      <td>0.287575</td>
      <td>0.234359</td>
      <td>0.257668</td>
      <td>0.000274</td>
      <td>0.350000</td>
      <td>0.172840</td>
      <td>0.255611</td>
      <td>0.003247</td>
      <td>0.615385</td>
      <td>0.239560</td>
      <td>0.044684</td>
      <td>0.307692</td>
      <td>0.076923</td>
      <td>0.169545</td>
      <td>0.006240</td>
      <td>0.615385</td>
      <td>0.239560</td>
      <td>0.044684</td>
      <td>6.616667</td>
      <td>3.563492</td>
      <td>5.083122</td>
      <td>0.950051</td>
      <td>66.992537</td>
      <td>28.248894</td>
      <td>47.236515</td>
      <td>150.711681</td>
      <td>4.0</td>
      <td>15.0</td>
      <td>15.0</td>
      <td>0.013600</td>
      <td>-2.688625</td>
      <td>15.0</td>
      <td>14.0</td>
      <td>4</td>
      <td>0</td>
      <td>0</td>
      <td>1</td>
      <td>1</td>
      <td>3</td>
      <td>0</td>
      <td>1</td>
      <td>1</td>
      <td>1</td>
      <td>1</td>
      <td>1</td>
      <td>1</td>
      <td>0</td>
      <td>0</td>
      <td>1</td>
      <td>1</td>
      <td>0</td>
      <td>1</td>
      <td>0</td>
      <td>1</td>
      <td>0</td>
      <td>0</td>
      <td>0</td>
    </tr>
  </tbody>
</table>

``` python
#using graphs, you can easily check whether two glycans are the same - even if they use different bracket notations!
from glycowork.motif.graph import compare_glycans
print(compare_glycans('Man(a1-3)[Man(a1-6)]Man(b1-4)GlcNAc(b1-4)[Fuc(a1-6)]GlcNAc',
                     'Man(a1-6)[Man(a1-3)]Man(b1-4)GlcNAc(b1-4)[Fuc(a1-6)]GlcNAc'))
print(compare_glycans('Man(a1-3)[Man(a1-6)]Man(b1-4)GlcNAc(b1-4)[Fuc(a1-6)]GlcNAc',
                     'Man(a1-6)[Man(a1-3)]Man(b1-4)GlcNAc(b1-4)GlcNAc'))

#or you could find the terminal structures of a glycan
from glycowork.motif.annotate import get_terminal_structures
print("\nTerminal structures:")
print(get_terminal_structures('Man(a1-3)[Man(a1-6)]Man(b1-4)GlcNAc(b1-4)[Fuc(a1-6)]GlcNAc'))
```

    True
    False

    Terminal structures:
    ['Man(a1-3)', 'Man(a1-6)', 'Fuc(a1-6)']

``` python
#given a composition, find matching glycan structures in sugarbase; can be set to species-specific structures
from glycowork.motif.tokenization import compositions_to_structures
print(compositions_to_structures([{'Hex':3, 'HexNAc':4}], reducing_end = 'GlcNAc'))

#or we could calculate the mass of this composition
from glycowork.motif.tokenization import composition_to_mass
print("\nMass of the composition Hex3HexNAc4")
print(composition_to_mass({'Hex':3, 'HexNAc':4}))
```

    0 compositions could not be matched. Run with verbose = True to see which compositions.
                                                  glycan  abundance
    0  GalNAc(b1-4)GlcNAc(b1-2)Man(a1-3)[Man(a1-6)]Ma...          0
    1  GlcNAc(?1-?)Man(a1-3)[GlcNAc(?1-?)Man(a1-6)]Ma...          0
    2  GlcNAc(b1-2)Man(a1-3)[GlcNAc(b1-6)Man(a1-6)]Ma...          0
    3  GlcNAc(b1-4)Man(a1-3)[GlcNAc(b1-6)Man(a1-6)]Ma...          0
    4  GlcNAc(b1-2)Man(a1-3)[GlcNAc(b1-2)Man(a1-6)]Ma...          0
    5  GlcNAc(b1-2)Man(a1-3)[GlcNAc(b1-2)Man(a1-6)][G...          0
    6  GlcNAc(b1-2)Man(a1-3)[GlcNAc(b1-2)[GlcNAc(b1-4...          0
    7  GlcNAc(b1-2)Man(a1-3)[GlcNAc(b1-4)][Man(a1-6)]...          0
    8  GlcNAc(b1-2)[GlcNAc(b1-4)]Man(a1-3)[GlcNAc(b1-...          0
    9  GlcNAc(b1-4)Man(a1-3)[GlcNAc(b1-2)Man(a1-6)]Ma...          0

    Mass of the composition Hex3HexNAc4
    1316.4865545999999
